package org.jeecg.modules.apply.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.apply.entity.ApplyLeave;
import org.jeecg.modules.apply.service.IApplyLeaveService;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: apply_leave
 * @Author: jeecg-boot
 * @Date:   2021-08-18
 * @Version: V1.0
 */
@Api(tags="apply_leave")
@RestController
@RequestMapping("/apply/applyLeave")
@Slf4j
public class ApplyLeaveController extends JeecgController<ApplyLeave, IApplyLeaveService> {
	@Autowired
	private IApplyLeaveService applyLeaveService;
	
	/**
	 * 分页列表查询
	 *
	 * @param applyLeave
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "apply_leave-分页列表查询")
	@ApiOperation(value="apply_leave-分页列表查询", notes="apply_leave-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(ApplyLeave applyLeave,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ApplyLeave> queryWrapper = QueryGenerator.initQueryWrapper(applyLeave, req.getParameterMap());
		Page<ApplyLeave> page = new Page<ApplyLeave>(pageNo, pageSize);
		IPage<ApplyLeave> pageList = applyLeaveService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param applyLeave
	 * @return
	 */
	@AutoLog(value = "apply_leave-添加")
	@ApiOperation(value="apply_leave-添加", notes="apply_leave-添加")
	@PostMapping(value = "/add")
	public Result<?> add(@RequestBody ApplyLeave applyLeave) {

		boolean b = applyLeaveService.saveLeave(applyLeave);

		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param applyLeave
	 * @return
	 */
	@AutoLog(value = "apply_leave-编辑")
	@ApiOperation(value="apply_leave-编辑", notes="apply_leave-编辑")
	@PutMapping(value = "/edit")
	public Result<?> edit(@RequestBody ApplyLeave applyLeave) {
		applyLeaveService.updateById(applyLeave);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "apply_leave-通过id删除")
	@ApiOperation(value="apply_leave-通过id删除", notes="apply_leave-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		applyLeaveService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "apply_leave-批量删除")
	@ApiOperation(value="apply_leave-批量删除", notes="apply_leave-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<?> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.applyLeaveService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "apply_leave-通过id查询")
	@ApiOperation(value="apply_leave-通过id查询", notes="apply_leave-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<?> queryById(@RequestParam(name="id",required=true) String id) {
		ApplyLeave applyLeave = applyLeaveService.getById(id);
		if(applyLeave==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(applyLeave);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param applyLeave
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ApplyLeave applyLeave) {
        return super.exportXls(request, applyLeave, ApplyLeave.class, "apply_leave");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) throws Exception {
        return super.importExcel(request, response, ApplyLeave.class);
    }

}
