package org.jeecg.modules.baogao.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.xkcoding.http.util.StringUtil;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.baogao.dto.ItdmBaogaoCreateDTO;
import org.jeecg.modules.baogao.entity.ItdmBaogao;
import org.jeecg.modules.baogao.service.IItdmBaogaoService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.baogao.vo.ItdmBaogaoVO;
import org.jeecg.modules.moban.entity.ItdmMoban;
import org.jeecg.modules.weituo.entity.ItdmWeituoInfo;
import org.jeecg.modules.weituo.entity.ItdmWeituoYangpin;
import org.jeecg.modules.weituo.entity.ItdmWeituoYangpinExtend;
import org.jeecg.modules.weituo.service.IItdmWeituoInfoService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

/**
 * @Description: 报告生成
 * @Author: jeecg-boot
 * @Date: 2023-05-19
 * @Version: V1.0
 */
@Api(tags = "报告生成")
@RestController
@RequestMapping("/baogao/itdmBaogao")
@Slf4j
public class ItdmBaogaoController extends JeecgController<ItdmBaogao, IItdmBaogaoService> {
    @Autowired
    private IItdmBaogaoService itdmBaogaoService;


    /**
     * 分页列表查询
     *
     * @param itdmBaogao
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "报告生成-分页列表查询")
    @ApiOperation(value = "报告生成-分页列表查询", notes = "报告生成-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ItdmBaogao>> queryPageList(ItdmBaogao itdmBaogao,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        QueryWrapper<ItdmBaogao> queryWrapper = QueryGenerator.initQueryWrapper(itdmBaogao, req.getParameterMap());
        Page<ItdmBaogao> page = new Page<ItdmBaogao>(pageNo, pageSize);
        IPage<ItdmBaogao> pageList = itdmBaogaoService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param itdmBaogao
     * @return
     */
    @AutoLog(value = "报告生成-添加")
    @ApiOperation(value = "报告生成-添加", notes = "报告生成-添加")
    //@RequiresPermissions("org.jeecg.modules:itdm_baogao:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody ItdmBaogaoCreateDTO itdmBaogao) {
        itdmBaogaoService.createBaogao(itdmBaogao);
        return Result.OK("添加成功！");
    }



    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "报告生成-通过id删除")
    @ApiOperation(value = "报告生成-通过id删除", notes = "报告生成-通过id删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_baogao:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        itdmBaogaoService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "报告生成-批量删除")
    @ApiOperation(value = "报告生成-批量删除", notes = "报告生成-批量删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_baogao:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.itdmBaogaoService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "报告生成-通过id查询")
    @ApiOperation(value = "报告生成-通过id查询", notes = "报告生成-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ItdmBaogao> queryById(@RequestParam(name = "id", required = true) String id) {
        ItdmBaogao itdmBaogao = itdmBaogaoService.getById(id);
        if (itdmBaogao == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmBaogao);
    }



    @ApiOperation(value = "onlyoffice 所需数据", notes = "onlyoffice 所需数据")


    @GetMapping(value = "/wordById")
    public  Result<ItdmBaogaoVO>  wordById(@RequestParam(name = "id", required = true) String id) {
        ItdmBaogaoVO itdmBaogao = itdmBaogaoService.wordById(id);

        if (itdmBaogao == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmBaogao);
    }



    @GetMapping(value = "/weituo/list")
    public Result<List<ItdmWeituoInfo>> queryweituoList() {
        List<ItdmWeituoInfo> pageList = itdmBaogaoService.queryweituoList();
        return Result.OK(pageList);
    }


    @GetMapping(value = "/yangpin/list")
    public Result<List<ItdmWeituoYangpin>> queryYangpinList(String weituoId) {
        if (StringUtil.isEmpty(weituoId)) {
            return Result.OK();
        }
        List<ItdmWeituoYangpin> pageList = itdmBaogaoService.queryYangpinList(weituoId);
        return Result.OK(pageList);
    }


    @GetMapping(value = "/shiyan/list")
    public Result<List<ItdmWeituoYangpinExtend>> queryShiyanList(String yangpinIds) {
        if (StringUtil.isEmpty(yangpinIds)) {
            return Result.OK();
        }
        List<ItdmWeituoYangpinExtend> pageList = itdmBaogaoService.queryShiyanList(Arrays.asList(yangpinIds.split(",")));
        return Result.OK(pageList);
    }



    @GetMapping(value = "/munam/queryById")
    public Result<ItdmMoban> queryMubanById(@RequestParam(name="mobanId",required=true) String mobanId) {
        ItdmMoban itdmMoban = itdmBaogaoService.queryMubanById(mobanId);
        if(itdmMoban==null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmMoban);
    }



}
