package org.jeecg.modules.btDemo.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.btDemo.entity.BtDemo;
import org.jeecg.modules.btDemo.service.IBtDemoService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 测试
 * @Author: jeecg-boot
 * @Date:   2023-05-15
 * @Version: V1.0
 */
@Api(tags="测试")
@RestController
@RequestMapping("/btDemo/btDemo")
@Slf4j
public class BtDemoController extends JeecgController<BtDemo, IBtDemoService> {
	@Autowired
	private IBtDemoService btDemoService;
	
	/**
	 * 分页列表查询
	 *
	 * @param btDemo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "测试-分页列表查询")
	@ApiOperation(value="测试-分页列表查询", notes="测试-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<BtDemo>> queryPageList(BtDemo btDemo,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<BtDemo> queryWrapper = QueryGenerator.initQueryWrapper(btDemo, req.getParameterMap());
		Page<BtDemo> page = new Page<BtDemo>(pageNo, pageSize);
		IPage<BtDemo> pageList = btDemoService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param btDemo
	 * @return
	 */
	@AutoLog(value = "测试-添加")
	@ApiOperation(value="测试-添加", notes="测试-添加")
	//@RequiresPermissions("org.jeecg.modules:bt_demo:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody BtDemo btDemo) {
		btDemoService.save(btDemo);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param btDemo
	 * @return
	 */
	@AutoLog(value = "测试-编辑")
	@ApiOperation(value="测试-编辑", notes="测试-编辑")
	//@RequiresPermissions("org.jeecg.modules:bt_demo:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody BtDemo btDemo) {
		btDemoService.updateById(btDemo);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "测试-通过id删除")
	@ApiOperation(value="测试-通过id删除", notes="测试-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:bt_demo:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		btDemoService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "测试-批量删除")
	@ApiOperation(value="测试-批量删除", notes="测试-批量删除")
	//@RequiresPermissions("org.jeecg.modules:bt_demo:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.btDemoService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "测试-通过id查询")
	@ApiOperation(value="测试-通过id查询", notes="测试-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<BtDemo> queryById(@RequestParam(name="id",required=true) String id) {
		BtDemo btDemo = btDemoService.getById(id);
		if(btDemo==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(btDemo);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param btDemo
    */
    //@RequiresPermissions("org.jeecg.modules:bt_demo:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, BtDemo btDemo) {
        return super.exportXls(request, btDemo, BtDemo.class, "测试");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("bt_demo:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, BtDemo.class);
    }

}
