package org.jeecg.modules.customform.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.customform.entity.CustomForm;
import org.jeecg.modules.customform.service.ICustomFormService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

/**
 * @Description: 自定义表单数据
 * @Author: jeecg-boot
 * @Date: 2023-06-21
 * @Version: V1.0
 */
@Api(tags = "自定义表单数据")
@RestController
@RequestMapping("/customform/customForm")
@Slf4j
public class CustomFormController extends JeecgController<CustomForm, ICustomFormService> {
    @Autowired
    private ICustomFormService customFormService;

    /**
     * 分页列表查询
     *
     * @param customForm
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "自定义表单数据-分页列表查询")
    @ApiOperation(value = "自定义表单数据-分页列表查询", notes = "自定义表单数据-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(CustomForm customForm,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
//		QueryWrapper<CustomForm> queryWrapper = QueryGenerator.initQueryWrapper(customForm, req.getParameterMap());
        QueryWrapper<CustomForm> queryWrapper = new QueryWrapper<>();
        if (customForm.getFormstatus() != null) {
            queryWrapper.like("formstatus", customForm.getFormstatus());
        }
        Page<CustomForm> page = new Page<CustomForm>(pageNo, pageSize);
        IPage<CustomForm> pageList = customFormService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param customForm
     * @return
     */
    @AutoLog(value = "自定义表单数据-添加")
    @ApiOperation(value = "自定义表单数据-添加", notes = "自定义表单数据-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody CustomForm customForm) {
        customFormService.save(customForm);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param customForm
     * @return
     */
    @AutoLog(value = "自定义表单数据-编辑")
    @ApiOperation(value = "自定义表单数据-编辑", notes = "自定义表单数据-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody CustomForm customForm) {
        customFormService.updateById(customForm);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "自定义表单数据-通过id删除")
    @ApiOperation(value = "自定义表单数据-通过id删除", notes = "自定义表单数据-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        customFormService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "自定义表单数据-批量删除")
    @ApiOperation(value = "自定义表单数据-批量删除", notes = "自定义表单数据-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.customFormService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "自定义表单数据-通过id查询")
    @ApiOperation(value = "自定义表单数据-通过id查询", notes = "自定义表单数据-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<?> queryById(@RequestParam(name = "id", required = true) String id) {
        CustomForm customForm = customFormService.getById(id);
        if (customForm == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(customForm);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param customForm
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, CustomForm customForm) {
        return super.exportXls(request, customForm, CustomForm.class, "自定义表单数据");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, CustomForm.class);
    }

}
