package org.jeecg.modules.flowpath.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.flowpath.entity.ItdmFlowPath;
import org.jeecg.modules.flowpath.service.IItdmFlowPathService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 流程表
 * @Author: jeecg-boot
 * @Date: 2023-07-20
 * @Version: V1.0
 */
@Api(tags = "流程表")
@RestController
@RequestMapping("/flowpath/itdmFlowPath")
@Slf4j
public class ItdmFlowPathController extends JeecgController<ItdmFlowPath, IItdmFlowPathService> {
    @Autowired
    private IItdmFlowPathService itdmFlowPathService;

    /**
     * 分页列表查询
     *
     * @param itdmFlowPath
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "流程表-分页列表查询")
    @ApiOperation(value = "流程表-分页列表查询", notes = "流程表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ItdmFlowPath>> queryPageList(ItdmFlowPath itdmFlowPath,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        QueryWrapper<ItdmFlowPath> queryWrapper = QueryGenerator.initQueryWrapper(itdmFlowPath, req.getParameterMap());
        Page<ItdmFlowPath> page = new Page<ItdmFlowPath>(pageNo, pageSize);
        IPage<ItdmFlowPath> pageList = itdmFlowPathService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    @GetMapping(value = "/querySelectList")
    public Result<List<Map<String, String>>> queryList() {
        LambdaQueryWrapper<ItdmFlowPath> queryWrapper = Wrappers.lambdaQuery(ItdmFlowPath.class).eq(ItdmFlowPath::getStatus, 0);
        List<Map<String, String>> pageList = itdmFlowPathService.list(queryWrapper).stream().map(
                i -> {
                    Map<String, String> map = new HashMap<>();
                    map.put("text", i.getName());
                    map.put("value", i.getId());
                    return map;
                }
        ).collect(Collectors.toList());
        return Result.OK(pageList);
    }


    /**
     * 添加
     *
     * @param itdmFlowPath
     * @return
     */
    @AutoLog(value = "流程表-添加")
    @ApiOperation(value = "流程表-添加", notes = "流程表-添加")
    //@RequiresPermissions("org.jeecg.modules:itdm_flow_path:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody ItdmFlowPath itdmFlowPath) {
        itdmFlowPathService.save(itdmFlowPath);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param itdmFlowPath
     * @return
     */
    @AutoLog(value = "流程表-编辑")
    @ApiOperation(value = "流程表-编辑", notes = "流程表-编辑")
    //@RequiresPermissions("org.jeecg.modules:itdm_flow_path:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody ItdmFlowPath itdmFlowPath) {
        itdmFlowPathService.updateById(itdmFlowPath);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "流程表-通过id删除")
    @ApiOperation(value = "流程表-通过id删除", notes = "流程表-通过id删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_flow_path:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        itdmFlowPathService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "流程表-批量删除")
    @ApiOperation(value = "流程表-批量删除", notes = "流程表-批量删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_flow_path:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.itdmFlowPathService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "流程表-通过id查询")
    @ApiOperation(value = "流程表-通过id查询", notes = "流程表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ItdmFlowPath> queryById(@RequestParam(name = "id", required = true) String id) {
        ItdmFlowPath itdmFlowPath = itdmFlowPathService.getById(id);
        if (itdmFlowPath == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmFlowPath);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param itdmFlowPath
     */
    //@RequiresPermissions("org.jeecg.modules:itdm_flow_path:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmFlowPath itdmFlowPath) {
        return super.exportXls(request, itdmFlowPath, ItdmFlowPath.class, "流程表");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("itdm_flow_path:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmFlowPath.class);
    }


}
