package org.jeecg.modules.flowpath.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.flowpath.entity.ItdmRunFlowPathLog;
import org.jeecg.modules.flowpath.service.IItdmRunFlowPathLogService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 运行流程步骤日志
 * @Author: jeecg-boot
 * @Date:   2023-07-20
 * @Version: V1.0
 */
@Api(tags="运行流程步骤日志")
@RestController
@RequestMapping("/flowpath/itdmRunFlowPathLog")
@Slf4j
public class ItdmRunFlowPathLogController extends JeecgController<ItdmRunFlowPathLog, IItdmRunFlowPathLogService> {
	@Autowired
	private IItdmRunFlowPathLogService itdmRunFlowPathLogService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmRunFlowPathLog
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "运行流程步骤日志-分页列表查询")
	@ApiOperation(value="运行流程步骤日志-分页列表查询", notes="运行流程步骤日志-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmRunFlowPathLog>> queryPageList(ItdmRunFlowPathLog itdmRunFlowPathLog,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmRunFlowPathLog> queryWrapper = QueryGenerator.initQueryWrapper(itdmRunFlowPathLog, req.getParameterMap());
		Page<ItdmRunFlowPathLog> page = new Page<ItdmRunFlowPathLog>(pageNo, pageSize);
		IPage<ItdmRunFlowPathLog> pageList = itdmRunFlowPathLogService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmRunFlowPathLog
	 * @return
	 */
	@AutoLog(value = "运行流程步骤日志-添加")
	@ApiOperation(value="运行流程步骤日志-添加", notes="运行流程步骤日志-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_run_flow_path_log:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmRunFlowPathLog itdmRunFlowPathLog) {
		itdmRunFlowPathLogService.save(itdmRunFlowPathLog);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmRunFlowPathLog
	 * @return
	 */
	@AutoLog(value = "运行流程步骤日志-编辑")
	@ApiOperation(value="运行流程步骤日志-编辑", notes="运行流程步骤日志-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_run_flow_path_log:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmRunFlowPathLog itdmRunFlowPathLog) {
		itdmRunFlowPathLogService.updateById(itdmRunFlowPathLog);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "运行流程步骤日志-通过id删除")
	@ApiOperation(value="运行流程步骤日志-通过id删除", notes="运行流程步骤日志-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_run_flow_path_log:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmRunFlowPathLogService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "运行流程步骤日志-批量删除")
	@ApiOperation(value="运行流程步骤日志-批量删除", notes="运行流程步骤日志-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_run_flow_path_log:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmRunFlowPathLogService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "运行流程步骤日志-通过id查询")
	@ApiOperation(value="运行流程步骤日志-通过id查询", notes="运行流程步骤日志-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmRunFlowPathLog> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmRunFlowPathLog itdmRunFlowPathLog = itdmRunFlowPathLogService.getById(id);
		if(itdmRunFlowPathLog==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmRunFlowPathLog);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmRunFlowPathLog
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_run_flow_path_log:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmRunFlowPathLog itdmRunFlowPathLog) {
        return super.exportXls(request, itdmRunFlowPathLog, ItdmRunFlowPathLog.class, "运行流程步骤日志");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_run_flow_path_log:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmRunFlowPathLog.class);
    }

}
