package org.jeecg.modules.inventory.controller;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.inventory.convert.ItdmSampleExpireRealConvert;
import org.jeecg.modules.inventory.entity.ItdmSampleExpireReal;
import org.jeecg.modules.inventory.entity.ItdmSampleInventory;
import org.jeecg.modules.inventory.service.IItdmSampleExpireRealService;
import org.jeecg.modules.inventory.service.IItdmSampleInventoryService;
import org.jeecg.modules.inventory.vo.ItdmSampleExpireRealListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 样品过期审核通过表
 * @Author: jeecg-boot
 * @Date:   2023-06-26
 * @Version: V1.0
 */
@Api(tags="样品过期审核通过表")
@RestController
@RequestMapping("/inventory/itdmSampleExpireReal")
@Slf4j
public class ItdmSampleExpireRealController extends JeecgController<ItdmSampleExpireReal, IItdmSampleExpireRealService> {
	@Autowired
	private IItdmSampleExpireRealService itdmSampleExpireRealService;
	@Autowired
	private IItdmSampleInventoryService sampleInventoryService;

	/**
	 * 分页列表查询
	 *
	 * @param itdmSampleExpireReal
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "样品过期审核通过表-分页列表查询")
	@ApiOperation(value="样品过期审核通过表-分页列表查询", notes="样品过期审核通过表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<Page<ItdmSampleExpireRealListVO>> queryPageList(ItdmSampleExpireReal itdmSampleExpireReal,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmSampleExpireReal> queryWrapper = QueryGenerator.initQueryWrapper(itdmSampleExpireReal, req.getParameterMap());
		Page<ItdmSampleExpireReal> page = new Page<ItdmSampleExpireReal>(pageNo, pageSize);
		Page<ItdmSampleExpireReal> pageList = itdmSampleExpireRealService.page(page, queryWrapper);
		if(pageList != null ){
			List<ItdmSampleExpireRealListVO> list = pageList.getRecords().stream().map(i->{
				ItdmSampleInventory inventory = new ItdmSampleInventory();
				if(i.getBarCode()!=null){
					inventory = sampleInventoryService.getByBarCode(i.getBarCode());
				}
				return ItdmSampleExpireRealConvert.INSTANCE.convert(i,inventory);
			}).collect(Collectors.toList());
			return Result.OK(ItdmSampleExpireRealConvert.INSTANCE.converter(pageList,list));
		}else return Result.OK(new Page<ItdmSampleExpireRealListVO>());

	}

	/**
	 *   添加
	 *
	 * @param itdmSampleExpireReal
	 * @return
	 */
	@AutoLog(value = "样品过期审核通过表-添加")
	@ApiOperation(value="样品过期审核通过表-添加", notes="样品过期审核通过表-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_sample_expire_real:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmSampleExpireReal itdmSampleExpireReal) {
		itdmSampleExpireRealService.save(itdmSampleExpireReal);
		return Result.OK("添加成功！");
	}

	/**
	 *  编辑
	 *
	 * @param itdmSampleExpireReal
	 * @return
	 */
	@AutoLog(value = "样品过期审核通过表-编辑")
	@ApiOperation(value="样品过期审核通过表-编辑", notes="样品过期审核通过表-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_sample_expire_real:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmSampleExpireReal itdmSampleExpireReal) {
		itdmSampleExpireRealService.updateById(itdmSampleExpireReal);
		return Result.OK("编辑成功!");
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "样品过期审核通过表-通过id删除")
	@ApiOperation(value="样品过期审核通过表-通过id删除", notes="样品过期审核通过表-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_sample_expire_real:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmSampleExpireRealService.removeById(id);
		return Result.OK("删除成功!");
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "样品过期审核通过表-批量删除")
	@ApiOperation(value="样品过期审核通过表-批量删除", notes="样品过期审核通过表-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_sample_expire_real:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmSampleExpireRealService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "样品过期审核通过表-通过id查询")
	@ApiOperation(value="样品过期审核通过表-通过id查询", notes="样品过期审核通过表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmSampleExpireReal> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmSampleExpireReal itdmSampleExpireReal = itdmSampleExpireRealService.getById(id);
		if(itdmSampleExpireReal==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmSampleExpireReal);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmSampleExpireReal
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_expire_real:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmSampleExpireReal itdmSampleExpireReal) {
        return super.exportXls(request, itdmSampleExpireReal, ItdmSampleExpireReal.class, "样品过期审核通过表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_sample_expire_real:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmSampleExpireReal.class);
    }

}
