package org.jeecg.modules.inventory.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.toolkit.Sequence;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.inventory.entity.ItdmSampleInventory;
import org.jeecg.modules.inventory.service.IItdmSampleInventoryService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

/**
 * @Description: 样品库存
 * @Author: jeecg-boot
 * @Date: 2023-06-25
 * @Version: V1.0
 */
@Api(tags = "样品库存")
@RestController
@RequestMapping("/inventory/itdmSampleInventory")
@Slf4j
public class ItdmSampleInventoryController extends JeecgController<ItdmSampleInventory, IItdmSampleInventoryService> {
    @Autowired
    private IItdmSampleInventoryService itdmSampleInventoryService;

    /**
     * 分页列表查询
     *
     * @param itdmSampleInventory
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "样品库存-分页列表查询")
    @ApiOperation(value = "样品库存-分页列表查询", notes = "样品库存-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ItdmSampleInventory>> queryPageList(ItdmSampleInventory itdmSampleInventory,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        QueryWrapper<ItdmSampleInventory> queryWrapper = QueryGenerator.initQueryWrapper(itdmSampleInventory, req.getParameterMap());
        Page<ItdmSampleInventory> page = new Page<ItdmSampleInventory>(pageNo, pageSize);
        IPage<ItdmSampleInventory> pageList = itdmSampleInventoryService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param itdmSampleInventory
     * @return
     */
    @AutoLog(value = "样品库存-添加")
    @ApiOperation(value = "样品库存-添加", notes = "样品库存-添加")
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_inventory:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody ItdmSampleInventory itdmSampleInventory) {
        itdmSampleInventory.setStatus("0");

        itdmSampleInventory.setSampleNum(0);
        itdmSampleInventory.setBatchNumber(new Sequence(1L, 1L).nextId() + "");
        itdmSampleInventoryService.save(itdmSampleInventory);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param itdmSampleInventory
     * @return
     */
    @AutoLog(value = "样品库存-编辑")
    @ApiOperation(value = "样品库存-编辑", notes = "样品库存-编辑")
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_inventory:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody ItdmSampleInventory itdmSampleInventory) {
        itdmSampleInventory.setSampleNum(null);
        itdmSampleInventory.setBatchNumber(null);
        itdmSampleInventory.setStatus(null);

        itdmSampleInventoryService.updateById(itdmSampleInventory);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "样品库存-通过id删除")
    @ApiOperation(value = "样品库存-通过id删除", notes = "样品库存-通过id删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_inventory:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        itdmSampleInventoryService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "样品库存-批量删除")
    @ApiOperation(value = "样品库存-批量删除", notes = "样品库存-批量删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_inventory:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.itdmSampleInventoryService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "样品库存-通过id查询")
    @ApiOperation(value = "样品库存-通过id查询", notes = "样品库存-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ItdmSampleInventory> queryById(@RequestParam(name = "id", required = true) String id) {
        ItdmSampleInventory itdmSampleInventory = itdmSampleInventoryService.getById(id);
        if (itdmSampleInventory == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmSampleInventory);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param itdmSampleInventory
     */
    //@RequiresPermissions("org.jeecg.modules:itdm_sample_inventory:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmSampleInventory itdmSampleInventory) {
        return super.exportXls(request, itdmSampleInventory, ItdmSampleInventory.class, "样品库存");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("itdm_sample_inventory:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmSampleInventory.class);
    }

}
