package org.jeecg.modules.itdmAlarm.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.itdmAlarm.entity.ItdmAlarm;
import org.jeecg.modules.itdmAlarm.service.IItdmAlarmService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 历史报警表
 * @Author: jeecg-boot
 * @Date:   2023-05-20
 * @Version: V1.0
 */
@Api(tags="历史报警表")
@RestController
@RequestMapping("/itdmAlarm/itdmAlarm")
@Slf4j
public class ItdmAlarmController extends JeecgController<ItdmAlarm, IItdmAlarmService> {
	@Autowired
	private IItdmAlarmService itdmAlarmService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmAlarm
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "历史报警表-分页列表查询")
	@ApiOperation(value="历史报警表-分页列表查询", notes="历史报警表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmAlarm>> queryPageList(ItdmAlarm itdmAlarm,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmAlarm> queryWrapper = QueryGenerator.initQueryWrapper(itdmAlarm, req.getParameterMap());
		Page<ItdmAlarm> page = new Page<ItdmAlarm>(pageNo, pageSize);
		IPage<ItdmAlarm> pageList = itdmAlarmService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmAlarm
	 * @return
	 */
	@AutoLog(value = "历史报警表-添加")
	@ApiOperation(value="历史报警表-添加", notes="历史报警表-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmAlarm itdmAlarm) {
		itdmAlarmService.save(itdmAlarm);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmAlarm
	 * @return
	 */
	@AutoLog(value = "历史报警表-编辑")
	@ApiOperation(value="历史报警表-编辑", notes="历史报警表-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmAlarm itdmAlarm) {
		itdmAlarmService.updateById(itdmAlarm);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "历史报警表-通过id删除")
	@ApiOperation(value="历史报警表-通过id删除", notes="历史报警表-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmAlarmService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "历史报警表-批量删除")
	@ApiOperation(value="历史报警表-批量删除", notes="历史报警表-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmAlarmService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "历史报警表-通过id查询")
	@ApiOperation(value="历史报警表-通过id查询", notes="历史报警表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmAlarm> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmAlarm itdmAlarm = itdmAlarmService.getById(id);
		if(itdmAlarm==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmAlarm);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmAlarm
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_alarm:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmAlarm itdmAlarm) {
        return super.exportXls(request, itdmAlarm, ItdmAlarm.class, "历史报警表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_alarm:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmAlarm.class);
    }

}
