package org.jeecg.modules.itdmAlarmTotal.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.itdmAlarmTotal.entity.ItdmAlarmTotal;
import org.jeecg.modules.itdmAlarmTotal.service.IItdmAlarmTotalService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 历史报警统计
 * @Author: jeecg-boot
 * @Date:   2023-05-20
 * @Version: V1.0
 */
@Api(tags="历史报警统计")
@RestController
@RequestMapping("/itdmAlarmTotal/itdmAlarmTotal")
@Slf4j
public class ItdmAlarmTotalController extends JeecgController<ItdmAlarmTotal, IItdmAlarmTotalService> {
	@Autowired
	private IItdmAlarmTotalService itdmAlarmTotalService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmAlarmTotal
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "历史报警统计-分页列表查询")
	@ApiOperation(value="历史报警统计-分页列表查询", notes="历史报警统计-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmAlarmTotal>> queryPageList(ItdmAlarmTotal itdmAlarmTotal,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmAlarmTotal> queryWrapper = QueryGenerator.initQueryWrapper(itdmAlarmTotal, req.getParameterMap());
		Page<ItdmAlarmTotal> page = new Page<ItdmAlarmTotal>(pageNo, pageSize);
		IPage<ItdmAlarmTotal> pageList = itdmAlarmTotalService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmAlarmTotal
	 * @return
	 */
	@AutoLog(value = "历史报警统计-添加")
	@ApiOperation(value="历史报警统计-添加", notes="历史报警统计-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm_total:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmAlarmTotal itdmAlarmTotal) {
		itdmAlarmTotalService.save(itdmAlarmTotal);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmAlarmTotal
	 * @return
	 */
	@AutoLog(value = "历史报警统计-编辑")
	@ApiOperation(value="历史报警统计-编辑", notes="历史报警统计-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm_total:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmAlarmTotal itdmAlarmTotal) {
		itdmAlarmTotalService.updateById(itdmAlarmTotal);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "历史报警统计-通过id删除")
	@ApiOperation(value="历史报警统计-通过id删除", notes="历史报警统计-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm_total:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmAlarmTotalService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "历史报警统计-批量删除")
	@ApiOperation(value="历史报警统计-批量删除", notes="历史报警统计-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_alarm_total:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmAlarmTotalService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "历史报警统计-通过id查询")
	@ApiOperation(value="历史报警统计-通过id查询", notes="历史报警统计-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmAlarmTotal> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmAlarmTotal itdmAlarmTotal = itdmAlarmTotalService.getById(id);
		if(itdmAlarmTotal==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmAlarmTotal);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmAlarmTotal
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_alarm_total:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmAlarmTotal itdmAlarmTotal) {
        return super.exportXls(request, itdmAlarmTotal, ItdmAlarmTotal.class, "历史报警统计");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_alarm_total:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmAlarmTotal.class);
    }

}
