package org.jeecg.modules.itdmDataHistory.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.itdmDataHistory.entity.ItdmEnvironmentQuyu;
import org.jeecg.modules.itdmDataHistory.service.IItdmEnvironmentQuyuService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 区域历史数据
 * @Author: jeecg-boot
 * @Date:   2023-05-30
 * @Version: V1.0
 */
@Api(tags="区域历史数据")
@RestController
@RequestMapping("/itdmDataHistory/itdmEnvironmentQuyu")
@Slf4j
public class ItdmEnvironmentQuyuController extends JeecgController<ItdmEnvironmentQuyu, IItdmEnvironmentQuyuService> {
	@Autowired
	private IItdmEnvironmentQuyuService itdmEnvironmentQuyuService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmEnvironmentQuyu
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "区域历史数据-分页列表查询")
	@ApiOperation(value="区域历史数据-分页列表查询", notes="区域历史数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmEnvironmentQuyu>> queryPageList(ItdmEnvironmentQuyu itdmEnvironmentQuyu,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmEnvironmentQuyu> queryWrapper = QueryGenerator.initQueryWrapper(itdmEnvironmentQuyu, req.getParameterMap());
		Page<ItdmEnvironmentQuyu> page = new Page<ItdmEnvironmentQuyu>(pageNo, pageSize);
		IPage<ItdmEnvironmentQuyu> pageList = itdmEnvironmentQuyuService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmEnvironmentQuyu
	 * @return
	 */
	@AutoLog(value = "区域历史数据-添加")
	@ApiOperation(value="区域历史数据-添加", notes="区域历史数据-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmEnvironmentQuyu itdmEnvironmentQuyu) {
		itdmEnvironmentQuyuService.save(itdmEnvironmentQuyu);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmEnvironmentQuyu
	 * @return
	 */
	@AutoLog(value = "区域历史数据-编辑")
	@ApiOperation(value="区域历史数据-编辑", notes="区域历史数据-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmEnvironmentQuyu itdmEnvironmentQuyu) {
		itdmEnvironmentQuyuService.updateById(itdmEnvironmentQuyu);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "区域历史数据-通过id删除")
	@ApiOperation(value="区域历史数据-通过id删除", notes="区域历史数据-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmEnvironmentQuyuService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "区域历史数据-批量删除")
	@ApiOperation(value="区域历史数据-批量删除", notes="区域历史数据-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmEnvironmentQuyuService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "区域历史数据-通过id查询")
	@ApiOperation(value="区域历史数据-通过id查询", notes="区域历史数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmEnvironmentQuyu> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmEnvironmentQuyu itdmEnvironmentQuyu = itdmEnvironmentQuyuService.getById(id);
		if(itdmEnvironmentQuyu==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmEnvironmentQuyu);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmEnvironmentQuyu
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmEnvironmentQuyu itdmEnvironmentQuyu) {
        return super.exportXls(request, itdmEnvironmentQuyu, ItdmEnvironmentQuyu.class, "区域历史数据");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_environment_quyu:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmEnvironmentQuyu.class);
    }

}
