package org.jeecg.modules.itdmDataNow.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.itdmDataNow.entity.ItdmEnvironmentQuyuNow;
import org.jeecg.modules.itdmDataNow.service.IItdmEnvironmentQuyuNowService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 区域实时数据
 * @Author: jeecg-boot
 * @Date:   2023-05-27
 * @Version: V1.0
 */
@Api(tags="区域实时数据")
@RestController
@RequestMapping("/itdmDataNow/itdmEnvironmentQuyuNow")
@Slf4j
public class ItdmEnvironmentQuyuNowController extends JeecgController<ItdmEnvironmentQuyuNow, IItdmEnvironmentQuyuNowService> {
	@Autowired
	private IItdmEnvironmentQuyuNowService itdmEnvironmentQuyuNowService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmEnvironmentQuyuNow
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "区域实时数据-分页列表查询")
	@ApiOperation(value="区域实时数据-分页列表查询", notes="区域实时数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmEnvironmentQuyuNow>> queryPageList(ItdmEnvironmentQuyuNow itdmEnvironmentQuyuNow,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmEnvironmentQuyuNow> queryWrapper = QueryGenerator.initQueryWrapper(itdmEnvironmentQuyuNow, req.getParameterMap());
		Page<ItdmEnvironmentQuyuNow> page = new Page<ItdmEnvironmentQuyuNow>(pageNo, pageSize);
		IPage<ItdmEnvironmentQuyuNow> pageList = itdmEnvironmentQuyuNowService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmEnvironmentQuyuNow
	 * @return
	 */
	@AutoLog(value = "区域实时数据-添加")
	@ApiOperation(value="区域实时数据-添加", notes="区域实时数据-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu_now:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmEnvironmentQuyuNow itdmEnvironmentQuyuNow) {
		itdmEnvironmentQuyuNowService.save(itdmEnvironmentQuyuNow);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmEnvironmentQuyuNow
	 * @return
	 */
	@AutoLog(value = "区域实时数据-编辑")
	@ApiOperation(value="区域实时数据-编辑", notes="区域实时数据-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu_now:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmEnvironmentQuyuNow itdmEnvironmentQuyuNow) {
		itdmEnvironmentQuyuNowService.updateById(itdmEnvironmentQuyuNow);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "区域实时数据-通过id删除")
	@ApiOperation(value="区域实时数据-通过id删除", notes="区域实时数据-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu_now:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmEnvironmentQuyuNowService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "区域实时数据-批量删除")
	@ApiOperation(value="区域实时数据-批量删除", notes="区域实时数据-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu_now:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmEnvironmentQuyuNowService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "区域实时数据-通过id查询")
	@ApiOperation(value="区域实时数据-通过id查询", notes="区域实时数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmEnvironmentQuyuNow> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmEnvironmentQuyuNow itdmEnvironmentQuyuNow = itdmEnvironmentQuyuNowService.getById(id);
		if(itdmEnvironmentQuyuNow==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmEnvironmentQuyuNow);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmEnvironmentQuyuNow
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_environment_quyu_now:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmEnvironmentQuyuNow itdmEnvironmentQuyuNow) {
        return super.exportXls(request, itdmEnvironmentQuyuNow, ItdmEnvironmentQuyuNow.class, "区域实时数据");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_environment_quyu_now:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmEnvironmentQuyuNow.class);
    }

}
