package org.jeecg.modules.itdmDeviceRun.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.itdmDeviceRun.entity.ItdmDeviceRun;
import org.jeecg.modules.itdmDeviceRun.service.IItdmDeviceRunService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 设备运行时间表
 * @Author: jeecg-boot
 * @Date:   2023-05-20
 * @Version: V1.0
 */
@Api(tags="设备运行时间表")
@RestController
@RequestMapping("/itdmDeviceRun/itdmDeviceRun")
@Slf4j
public class ItdmDeviceRunController extends JeecgController<ItdmDeviceRun, IItdmDeviceRunService> {
	@Autowired
	private IItdmDeviceRunService itdmDeviceRunService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmDeviceRun
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "设备运行时间表-分页列表查询")
	@ApiOperation(value="设备运行时间表-分页列表查询", notes="设备运行时间表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmDeviceRun>> queryPageList(ItdmDeviceRun itdmDeviceRun,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmDeviceRun> queryWrapper = QueryGenerator.initQueryWrapper(itdmDeviceRun, req.getParameterMap());
		Page<ItdmDeviceRun> page = new Page<ItdmDeviceRun>(pageNo, pageSize);
		IPage<ItdmDeviceRun> pageList = itdmDeviceRunService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmDeviceRun
	 * @return
	 */
	@AutoLog(value = "设备运行时间表-添加")
	@ApiOperation(value="设备运行时间表-添加", notes="设备运行时间表-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_device_run:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmDeviceRun itdmDeviceRun) {
		itdmDeviceRunService.save(itdmDeviceRun);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmDeviceRun
	 * @return
	 */
	@AutoLog(value = "设备运行时间表-编辑")
	@ApiOperation(value="设备运行时间表-编辑", notes="设备运行时间表-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_device_run:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmDeviceRun itdmDeviceRun) {
		itdmDeviceRunService.updateById(itdmDeviceRun);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "设备运行时间表-通过id删除")
	@ApiOperation(value="设备运行时间表-通过id删除", notes="设备运行时间表-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_device_run:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmDeviceRunService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "设备运行时间表-批量删除")
	@ApiOperation(value="设备运行时间表-批量删除", notes="设备运行时间表-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_device_run:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmDeviceRunService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "设备运行时间表-通过id查询")
	@ApiOperation(value="设备运行时间表-通过id查询", notes="设备运行时间表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmDeviceRun> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmDeviceRun itdmDeviceRun = itdmDeviceRunService.getById(id);
		if(itdmDeviceRun==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmDeviceRun);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmDeviceRun
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_device_run:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmDeviceRun itdmDeviceRun) {
        return super.exportXls(request, itdmDeviceRun, ItdmDeviceRun.class, "设备运行时间表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_device_run:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmDeviceRun.class);
    }

}
