package org.jeecg.modules.itdmGongDan.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.itdmGongDan.entity.ItdmGongdanMaster;
import org.jeecg.modules.itdmGongDan.service.IItdmGongdanMasterService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 工单master
 * @Author: jeecg-boot
 * @Date:   2023-05-28
 * @Version: V1.0
 */
@Api(tags="工单master")
@RestController
@RequestMapping("/itdmGongdanMaster/itdmGongdanMaster")
@Slf4j
public class ItdmGongdanMasterController extends JeecgController<ItdmGongdanMaster, IItdmGongdanMasterService> {
	@Autowired
	private IItdmGongdanMasterService itdmGongdanMasterService;

	/**
	 * 分页列表查询
	 *
	 * @param itdmGongdanMaster
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "工单master-分页列表查询")
	@ApiOperation(value="工单master-分页列表查询", notes="工单master-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmGongdanMaster>> queryPageList(ItdmGongdanMaster itdmGongdanMaster,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmGongdanMaster> queryWrapper = QueryGenerator.initQueryWrapper(itdmGongdanMaster, req.getParameterMap());
		Page<ItdmGongdanMaster> page = new Page<ItdmGongdanMaster>(pageNo, pageSize);
		IPage<ItdmGongdanMaster> pageList = itdmGongdanMasterService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param itdmGongdanMaster
	 * @return
	 */
	@AutoLog(value = "工单master-添加")
	@ApiOperation(value="工单master-添加", notes="工单master-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_gongdan_master:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmGongdanMaster itdmGongdanMaster) {
		itdmGongdanMasterService.save(itdmGongdanMaster);
		return Result.OK("添加成功！");
	}

	/**
	 *  编辑
	 *
	 * @param itdmGongdanMaster
	 * @return
	 */
	@AutoLog(value = "工单master-编辑")
	@ApiOperation(value="工单master-编辑", notes="工单master-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_gongdan_master:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmGongdanMaster itdmGongdanMaster) {
		itdmGongdanMasterService.updateById(itdmGongdanMaster);
		return Result.OK("编辑成功!");
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "工单master-通过id删除")
	@ApiOperation(value="工单master-通过id删除", notes="工单master-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_gongdan_master:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmGongdanMasterService.removeById(id);
		return Result.OK("删除成功!");
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "工单master-批量删除")
	@ApiOperation(value="工单master-批量删除", notes="工单master-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_gongdan_master:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmGongdanMasterService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "工单master-通过id查询")
	@ApiOperation(value="工单master-通过id查询", notes="工单master-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmGongdanMaster> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmGongdanMaster itdmGongdanMaster = itdmGongdanMasterService.getById(id);
		if(itdmGongdanMaster==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmGongdanMaster);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmGongdanMaster
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_gongdan_master:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmGongdanMaster itdmGongdanMaster) {
        return super.exportXls(request, itdmGongdanMaster, ItdmGongdanMaster.class, "工单master");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_gongdan_master:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmGongdanMaster.class);
    }

}
