package org.jeecg.modules.itdmGongDan.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.itdmGongDan.entity.ItdmTestPlan;
import org.jeecg.modules.itdmGongDan.service.IItdmTestPlanService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 试验计划
 * @Author: jeecg-boot
 * @Date:   2023-05-21
 * @Version: V1.0
 */
@Api(tags="试验计划")
@RestController
@RequestMapping("/itdmTestPlan/itdmTestPlan")
@Slf4j
public class ItdmTestPlanController extends JeecgController<ItdmTestPlan, IItdmTestPlanService> {
	@Autowired
	private IItdmTestPlanService itdmTestPlanService;

	/**
	 * 分页列表查询
	 *
	 * @param itdmTestPlan
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "试验计划-分页列表查询")
	@ApiOperation(value="试验计划-分页列表查询", notes="试验计划-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmTestPlan>> queryPageList(ItdmTestPlan itdmTestPlan,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmTestPlan> queryWrapper = QueryGenerator.initQueryWrapper(itdmTestPlan, req.getParameterMap());
		Page<ItdmTestPlan> page = new Page<ItdmTestPlan>(pageNo, pageSize);
		IPage<ItdmTestPlan> pageList = itdmTestPlanService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param itdmTestPlan
	 * @return
	 */
	@AutoLog(value = "试验计划-添加")
	@ApiOperation(value="试验计划-添加", notes="试验计划-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_test_plan:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmTestPlan itdmTestPlan) {
		itdmTestPlanService.save(itdmTestPlan);
		return Result.OK("添加成功！");
	}

	/**
	 *  编辑
	 *
	 * @param itdmTestPlan
	 * @return
	 */
	@AutoLog(value = "试验计划-编辑")
	@ApiOperation(value="试验计划-编辑", notes="试验计划-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_test_plan:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmTestPlan itdmTestPlan) {
		itdmTestPlanService.updateById(itdmTestPlan);
		return Result.OK("编辑成功!");
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "试验计划-通过id删除")
	@ApiOperation(value="试验计划-通过id删除", notes="试验计划-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_test_plan:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmTestPlanService.removeById(id);
		return Result.OK("删除成功!");
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "试验计划-批量删除")
	@ApiOperation(value="试验计划-批量删除", notes="试验计划-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_test_plan:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmTestPlanService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "试验计划-通过id查询")
	@ApiOperation(value="试验计划-通过id查询", notes="试验计划-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmTestPlan> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmTestPlan itdmTestPlan = itdmTestPlanService.getById(id);
		if(itdmTestPlan==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmTestPlan);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmTestPlan
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_test_plan:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmTestPlan itdmTestPlan) {
        return super.exportXls(request, itdmTestPlan, ItdmTestPlan.class, "试验计划");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_test_plan:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmTestPlan.class);
    }

}
