package org.jeecg.modules.itdmPersonManage.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.itdmPersonManage.entity.ItdmPersonManage;
import org.jeecg.modules.itdmPersonManage.service.IItdmPersonManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: 人员管理
 * @Author: jeecg-boot
 * @Date:   2023-05-17
 * @Version: V1.0
 */
@Api(tags="人员管理")
@RestController
@RequestMapping("/itdmPersonManage/itdmPersonManage")
@Slf4j
public class ItdmPersonManageController extends JeecgController<ItdmPersonManage, IItdmPersonManageService> {
	@Autowired
	private IItdmPersonManageService itdmPersonManageService;

	 /**
	  * 项目经理、操作员下拉列表————不用了
	  */
	 @ApiOperation(value="项目经理、操作员下拉列表————不用了", notes="目经理、操作员下拉列表————不用了")
	 @GetMapping("/xialalist")
	 public List<ItdmPersonManage> list1(@RequestParam("position") String position)
	 {
		 List<ItdmPersonManage> list = itdmPersonManageService.queryList(position);
		 return list;
	 }


	/**
	 * 分页列表查询
	 *
	 * @param itdmPersonManage
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "人员管理-分页列表查询")
	@ApiOperation(value="人员管理-分页列表查询", notes="人员管理-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmPersonManage>> queryPageList(ItdmPersonManage itdmPersonManage,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmPersonManage> queryWrapper = QueryGenerator.initQueryWrapper(itdmPersonManage, req.getParameterMap());
		Page<ItdmPersonManage> page = new Page<ItdmPersonManage>(pageNo, pageSize);
		IPage<ItdmPersonManage> pageList = itdmPersonManageService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param itdmPersonManage
	 * @return
	 */
	@AutoLog(value = "人员管理-添加")
	@ApiOperation(value="人员管理-添加", notes="人员管理-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_person_manage:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmPersonManage itdmPersonManage) {
		itdmPersonManageService.save(itdmPersonManage);
		return Result.OK("添加成功！");
	}

	/**
	 *  编辑
	 *
	 * @param itdmPersonManage
	 * @return
	 */
	@AutoLog(value = "人员管理-编辑")
	@ApiOperation(value="人员管理-编辑", notes="人员管理-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_person_manage:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmPersonManage itdmPersonManage) {
		itdmPersonManageService.updateById(itdmPersonManage);
		return Result.OK("编辑成功!");
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "人员管理-通过id删除")
	@ApiOperation(value="人员管理-通过id删除", notes="人员管理-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_person_manage:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmPersonManageService.removeById(id);
		return Result.OK("删除成功!");
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "人员管理-批量删除")
	@ApiOperation(value="人员管理-批量删除", notes="人员管理-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_person_manage:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmPersonManageService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "人员管理-通过id查询")
	@ApiOperation(value="人员管理-通过id查询", notes="人员管理-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmPersonManage> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmPersonManage itdmPersonManage = itdmPersonManageService.getById(id);
		if(itdmPersonManage==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmPersonManage);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmPersonManage
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_person_manage:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmPersonManage itdmPersonManage) {
        return super.exportXls(request, itdmPersonManage, ItdmPersonManage.class, "人员管理");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_person_manage:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmPersonManage.class);
    }

}
