package org.jeecg.modules.moban.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.moban.entity.ItdmMoban;
import org.jeecg.modules.moban.service.IItdmMobanService;
import org.jeecg.modules.moban.vo.IItdmMobanVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

 /**
 * @Description: 模板
 * @Author: jeecg-boot
 * @Date:   2023-05-19
 * @Version: V1.0
 */
@Api(tags="模板")
@RestController
@RequestMapping("/moban/itdmMoban")
@Slf4j
public class ItdmMobanController extends JeecgController<ItdmMoban, IItdmMobanService> {
	@Autowired
	private IItdmMobanService itdmMobanService;
	
	/**
	 * 分页列表查询
	 *
	 * @param itdmMoban
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "模板-分页列表查询")
	@ApiOperation(value="模板-分页列表查询", notes="模板-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmMoban>> queryPageList(ItdmMoban itdmMoban,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<ItdmMoban> queryWrapper = QueryGenerator.initQueryWrapper(itdmMoban, req.getParameterMap());
		Page<ItdmMoban> page = new Page<ItdmMoban>(pageNo, pageSize);
		IPage<ItdmMoban> pageList = itdmMobanService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param itdmMoban
	 * @return
	 */
	@AutoLog(value = "模板-添加")
	@ApiOperation(value="模板-添加", notes="模板-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_moban:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmMoban itdmMoban) {
		itdmMobanService.save(itdmMoban);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param itdmMoban
	 * @return
	 */
	@AutoLog(value = "模板-编辑")
	@ApiOperation(value="模板-编辑", notes="模板-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_moban:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmMoban itdmMoban) {
		itdmMobanService.updateById(itdmMoban);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "模板-通过id删除")
	@ApiOperation(value="模板-通过id删除", notes="模板-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_moban:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmMobanService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "模板-批量删除")
	@ApiOperation(value="模板-批量删除", notes="模板-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_moban:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmMobanService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "模板-通过id查询")
	@ApiOperation(value="模板-通过id查询", notes="模板-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmMoban> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmMoban itdmMoban = itdmMobanService.getById(id);
		if(itdmMoban==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmMoban);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmMoban
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_moban:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmMoban itdmMoban) {
        return super.exportXls(request, itdmMoban, ItdmMoban.class, "模板");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_moban:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmMoban.class);
    }








	 @ApiOperation(value = "onlyoffice 所需数据", notes = "onlyoffice 所需数据")


	 @GetMapping(value = "/wordById")
	 public  Result<IItdmMobanVO>  wordById(@RequestParam(name = "id", required = true) String id) {
		 IItdmMobanVO itdmBaogao = itdmMobanService.wordById(id);

		 if (itdmBaogao == null) {
			 return Result.error("未找到对应数据");
		 }
		 return Result.OK(itdmBaogao);
	 }

}
