package org.jeecg.modules.weituo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.itdmFirstPage.entity.ItdmTask;
import org.jeecg.modules.itdmFirstPage.service.IItdmTaskService;
import org.jeecg.modules.itdmGongDan.service.IItdmJCXMSHJHService;
import org.jeecg.modules.qita.GlobalVariables;
import org.jeecg.modules.system.mapper.SysUserMapper;
import org.jeecg.modules.weituo.dto.WeituoInsertCommand;
import org.jeecg.modules.weituo.dto.WeituoUpdateCommand;
import org.jeecg.modules.weituo.entity.ItdmWeituoInfo;
import org.jeecg.modules.weituo.entity.ItdmWeituoJieXaingVo;
import org.jeecg.modules.weituo.entity.ItdmWeituoYangpin;
import org.jeecg.modules.weituo.mapper.ItdmWeituoYangpinMapper;
import org.jeecg.modules.weituo.service.IItdmWeituoInfoService;
import org.jeecg.modules.weituo.vo.ItdmWeituoInfoInfoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description: 委托信息
 * @Author: jeecg-boot
 * @Date: 2023-05-17
 * @Version: V1.0
 */
@Api(tags = "委托信息")
@RestController
@RequestMapping("/weituo/itdmWeituoInfo")
@Slf4j
public class ItdmWeituoInfoController extends JeecgController<ItdmWeituoInfo, IItdmWeituoInfoService> {
    @Autowired
    private IItdmWeituoInfoService itdmWeituoInfoService;
    @Autowired
    private IItdmTaskService taskService;
    @Autowired
    @SuppressWarnings("all")
    private ItdmWeituoYangpinMapper itdmWeituoYangpinMapper;

    @Autowired
    private IItdmJCXMSHJHService itdmJCXMSHJHService;
    @Autowired
    @SuppressWarnings("all")
    private SysUserMapper sysUserMapper;


    /**
     * 判断委托完成
     */
    @AutoLog(value = "判断委托完成")
    @ApiOperation(value = "判断委托完成", notes = "判断委托完成")
    @RequestMapping(value = "/edit1", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit1(@RequestBody ItdmWeituoInfo itdmWeituoInfo) {
        itdmWeituoInfoService.updateById(itdmWeituoInfo);
        return Result.OK("编辑成功!");
    }

    @ApiOperation(value = "分页查询已审核通过的列表", notes = "分页查询已审核通过的列表")
    @GetMapping(value = "/SHTGlist")
    public Result<IPage<ItdmWeituoInfo>> querySHTGPageList(ItdmWeituoInfo itdmWeituoInfo,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        itdmWeituoInfo.setShenheStatus(String.valueOf(1));//审核通过
        QueryWrapper<ItdmWeituoInfo> queryWrapper = QueryGenerator.initQueryWrapper(itdmWeituoInfo, req.getParameterMap());
        Page<ItdmWeituoInfo> page = new Page<ItdmWeituoInfo>(pageNo, pageSize);
        IPage<ItdmWeituoInfo> pageList = itdmWeituoInfoService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 根据委托单位查询委托信息列表
     */
    @ApiOperation(value = "根据委托单位查询委托信息列表", notes = "根据委托单位查询委托信息列表")
    @GetMapping(value = "/weituoNoList")
    public Result<List<ItdmWeituoInfo>> selectWTListByClient(@RequestParam(name = "weituoClient") String weituoClient) {
        return Result.OK(itdmWeituoInfoService.selectWTListByClient(weituoClient));
    }

    /**
     * 查询委托单位名称（不重复）
     */
    @ApiOperation(value = "查询委托单位列表", notes = "查询委托单位列表")
    @GetMapping(value = "/distinctClientList")
    public Result<List<String>> selectDistinctClientList() {
        return Result.OK(itdmWeituoInfoService.selectDistinctClientList());
    }


    /**
    *   author: dzc
    *   version: 1.0
    *   des: 委托登记的查询
    *   date: 2023/11/10
    */
    @ApiOperation(value = "委托信息-分页列表查询", notes = "委托信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ItdmWeituoInfo>> queryWeituoDengJiPageList(ItdmWeituoInfo itdmWeituoInfo,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        QueryWrapper<ItdmWeituoInfo> queryWrapper = QueryGenerator.initQueryWrapper(itdmWeituoInfo, req.getParameterMap());
        Page<ItdmWeituoInfo> page = new Page<ItdmWeituoInfo>(pageNo, pageSize);
        IPage<ItdmWeituoInfo> pageList = itdmWeituoInfoService.page(page, queryWrapper);
        return Result.ok(pageList);
    }


    /**
     * 分页列表查询
     *
     * @param itdmWeituoInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "委托信息-分页列表查询")
    @ApiOperation(value = "委托结项-分页列表查询", notes = "委托结项-分页列表查询")
    @GetMapping(value = "/list1")
    public Result<IPage<ItdmWeituoJieXaingVo>> queryPageList(ItdmWeituoInfo itdmWeituoInfo,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        itdmWeituoInfo.setShenheStatus("1");//只显示审核通过的委托
        Page<ItdmWeituoJieXaingVo> page = new Page<ItdmWeituoJieXaingVo>(pageNo, pageSize);
        List<ItdmWeituoJieXaingVo> weituoList = itdmWeituoInfoService.getWeituoList(page, itdmWeituoInfo);
        page.setRecords(weituoList);
        return Result.OK(page);
    }

    /**
     * 添加——提交
     *
     * @param itdmWeituoInfo
     * @return
     */
    @AutoLog(value = "委托信息-添加")
    @ApiOperation(value = "委托信息-添加", notes = "委托信息-添加")
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody WeituoInsertCommand itdmWeituoInfo) {
        itdmWeituoInfoService.saveWeituo(itdmWeituoInfo);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑——提交
     *
     * @param itdmWeituoInfo
     * @return
     */
    @AutoLog(value = "委托信息-编辑")
    @ApiOperation(value = "委托信息-编辑", notes = "委托信息-编辑")
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody WeituoUpdateCommand itdmWeituoInfo) {
        itdmWeituoInfoService.updateWeituo(itdmWeituoInfo);

        return Result.OK("编辑成功!");
    }

    @AutoLog(value = "委托信息-暂存")
    @ApiOperation(value = "委托信息-暂存", notes = "委托信息-暂存")
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:add")
    @PostMapping(value = "/staging")
    public Result<String> Staging(@RequestBody WeituoUpdateCommand itdmWeituoInfo) {
        itdmWeituoInfoService.stagingWeituo(itdmWeituoInfo);
        return Result.OK("暂存成功！");
    }
    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "委托信息-通过id删除")
    @ApiOperation(value = "委托信息-通过id删除", notes = "委托信息-通过id删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        itdmWeituoInfoService.deleteById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "委托信息-批量删除")
    @ApiOperation(value = "委托信息-批量删除", notes = "委托信息-批量删除")
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.itdmWeituoInfoService.deleteByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "委托信息-通过id查询")
    @ApiOperation(value = "委托信息-通过id查询", notes = "委托信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ItdmWeituoInfoInfoVO> queryById(@RequestParam(name = "id", required = true) String id) {
        ItdmWeituoInfoInfoVO itdmWeituoInfo = itdmWeituoInfoService.findById(id);
        if (itdmWeituoInfo == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(itdmWeituoInfo);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param itdmWeituoInfo
     */
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_info:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmWeituoInfo itdmWeituoInfo) {
        return super.exportXls(request, itdmWeituoInfo, ItdmWeituoInfo.class, "委托信息");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("itdm_weituo_info:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmWeituoInfo.class);
    }


    /**
     * 审核通过或拒绝
     */
    @AutoLog(value = "审核通过或拒绝")
    @ApiOperation(value = "审核通过或拒绝", notes = "审核通过或拒绝")
    @RequestMapping("/saveOrUpdateWeituo")
    public Result<String> saveOrUpdateWeituo(@RequestBody ItdmWeituoInfo weituoInfo) {
        try {
            if (StringUtils.isBlank(weituoInfo.getId())) { // 添加
                weituoInfo.setCreateTime(new Date());
                weituoInfo.setShenheStatus("待审核"); // 初始状态为待审核
                itdmWeituoInfoService.save(weituoInfo); // 调用 service 的保存方法
            } else { // 编辑
                ItdmWeituoInfo oldWeituo = itdmWeituoInfoService.getById(weituoInfo.getId());
                if (oldWeituo == null) {
                    return Result.error("未找到对应数据");
                }
                // 这里只修改一些需要修改的属性，其他属性不变
                oldWeituo.setShenheStatus(weituoInfo.getShenheStatus());
                oldWeituo.setShenheMsg(weituoInfo.getShenheMsg());
                oldWeituo.setWeituoNo(weituoInfo.getWeituoNo());
                oldWeituo.setShenheTime((new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date())));

                // 需要根据业务逻辑进行判断
                if ("1".equals(oldWeituo.getShenheStatus())) {
                    oldWeituo.setUpdateTime(new Date());
                }

                itdmWeituoInfoService.updateById(oldWeituo); // 调用 service 的更新方法

                //新增task
                if (weituoInfo.getShenheStatus().equals("1")) { //审核通过

                    List<ItdmWeituoYangpin> yangpins = itdmWeituoYangpinMapper.selectList(Wrappers.lambdaQuery(ItdmWeituoYangpin.class).eq(ItdmWeituoYangpin::getWeituoId, weituoInfo.getId()));


                    int i = 1;


                    for (ItdmWeituoYangpin yangpin : yangpins) {

                        Integer sampleQuantities = Integer.parseInt(yangpin.getSampleQuantities());
                        String componentNo = sampleQuantities > 1 ?
                                weituoInfo.getWeituoNo() + String.format("-%03d~", i) + String.format("%03d", i + sampleQuantities - 1) :
                                weituoInfo.getWeituoNo() + String.format("-%03d", i);
                        i = i + sampleQuantities;
                        yangpin.setSampleCode(componentNo);
                        itdmWeituoYangpinMapper.updateById(yangpin);


                    }

                    //新增所有计划员计划排期的task
                    List<String> userNameList = sysUserMapper.getUserNameByRoleName(GlobalVariables.jhy);//计划员
                    for (String userName : userNameList){
                        ItdmTask itdmTask = new ItdmTask();
                        itdmTask.setTaskRole(GlobalVariables.jhy);//计划员
                        itdmTask.setTaskStatus(0);//待办
                        itdmTask.setWeituoId(weituoInfo.getId());
                        itdmTask.setTaskContent("计划排期");
                        itdmTask.setTaskUser(userName);
                        taskService.save(itdmTask);
                    }

                    LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
                    String realName = loginUser.getRealname(); // 登录人姓名

                    //修改该委托单该经理的task，状态改为1进行中
                    Map<String, Object> columnMap = new HashMap<>();
                    columnMap.put("task_role", GlobalVariables.jl);//”经理“
                    columnMap.put("weituo_id", weituoInfo.getId());
                    columnMap.put("task_user", realName);
                    List<ItdmTask> taskList = taskService.listByMap(columnMap);
                    if (taskList != null) {
                        ItdmTask itdmTask1 = taskList.get(0);
                        itdmTask1.setTaskStatus(1);
                        taskService.updateById(itdmTask1);
                    }

                    //删除该委托单其他经理状态为待办的task
                    Map<String, Object> deleteMap = new HashMap<>();
                    deleteMap.put("task_role",GlobalVariables.jl);//”经理“
                    columnMap.put("weituo_id", weituoInfo.getId());
                    deleteMap.put("task_status",0);
                    taskService.removeByMap(deleteMap);



                }else if (weituoInfo.getShenheStatus().equals("2")) { //审核拒绝
                    //删掉所有该委托id对应的“审核”的任务
                    Map<String, Object> deleteMap = new HashMap<>();
                    deleteMap.put("weituo_id", weituoInfo.getId());
                    deleteMap.put("task_role",GlobalVariables.jl);//”经理“
                    deleteMap.put("task_status",0);
                    taskService.removeByMap(deleteMap);

                }
            }

            return Result.OK("操作成功！");
        } catch (Exception e) {
            log.error("添加或编辑委托信息出错！", e);
            return Result.error("操作失败！");
        }
    }


    @GetMapping("/word")
    @ApiOperation(value = "委托单导出", notes = "委托单导出")

    public void word(String id, HttpServletResponse response) throws IOException {
        itdmWeituoInfoService.word(id, response);
    }



    /**
     * 根据委托单位查询委托信息列表
     */
    @ApiOperation(value = "根据委托单位查询委托信息列表", notes = "根据委托单位查询委托信息列表")
    @GetMapping(value = "/shenheList")
    public Result<List<ItdmWeituoInfo>> shenheList() {
        return Result.OK(itdmWeituoInfoService.shenheList());
    }


}
