package org.jeecg.modules.weituoyiju.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.weituoyiju.entity.ItdmWeituoYiju;
import org.jeecg.modules.weituoyiju.service.IItdmWeituoYijuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 委托依据
 * @Author: jeecg-boot
 * @Date:   2023-05-17
 * @Version: V1.0
 */
@Api(tags="委托依据")
@RestController
@RequestMapping("/weituoyiju/itdmWeituoYiju")
@Slf4j
public class ItdmWeituoYijuController extends JeecgController<ItdmWeituoYiju, IItdmWeituoYijuService> {
	@Autowired
	private IItdmWeituoYijuService itdmWeituoYijuService;

	/**
	 * 分页列表查询
	 *
	 * @param itdmWeituoYiju
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "委托依据-分页列表查询")
	@ApiOperation(value="委托依据-分页列表查询", notes="委托依据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ItdmWeituoYiju>> queryPageList(ItdmWeituoYiju itdmWeituoYiju,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		ItdmWeituoYiju itdmWeituoYiju1 = new ItdmWeituoYiju();
		QueryWrapper<ItdmWeituoYiju> queryWrapper = QueryGenerator.initQueryWrapper(itdmWeituoYiju1, req.getParameterMap());
		queryWrapper.lambda().like(StringUtils.isNoneBlank(itdmWeituoYiju.getCore()),ItdmWeituoYiju::getCore,itdmWeituoYiju.getCore());
		queryWrapper.lambda().like(StringUtils.isNoneBlank(itdmWeituoYiju.getName()),ItdmWeituoYiju::getName,itdmWeituoYiju.getName());
		Page<ItdmWeituoYiju> page = new Page<ItdmWeituoYiju>(pageNo, pageSize);
		IPage<ItdmWeituoYiju> pageList = itdmWeituoYijuService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param itdmWeituoYiju
	 * @return
	 */
	@AutoLog(value = "委托依据-添加")
	@ApiOperation(value="委托依据-添加", notes="委托依据-添加")
	//@RequiresPermissions("org.jeecg.modules:itdm_weituo_yiju:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody ItdmWeituoYiju itdmWeituoYiju) {
		Map<String, Object> columnMap = new HashMap<>();
		columnMap.put("name",itdmWeituoYiju.getName());
		List<ItdmWeituoYiju> list = itdmWeituoYijuService.listByMap(columnMap);
		if(list!=null && !list.isEmpty()){
			return Result.error("该检测依据 "+ itdmWeituoYiju.getName()+ " 已存在！");
		}else {
			itdmWeituoYijuService.save(itdmWeituoYiju);
			return Result.OK("添加成功！");
		}

	}

	/**
	 *  编辑
	 *
	 * @param itdmWeituoYiju
	 * @return
	 */
	@AutoLog(value = "委托依据-编辑")
	@ApiOperation(value="委托依据-编辑", notes="委托依据-编辑")
	//@RequiresPermissions("org.jeecg.modules:itdm_weituo_yiju:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody ItdmWeituoYiju itdmWeituoYiju) {
		ItdmWeituoYiju itdmWeituoYiju1 = itdmWeituoYijuService.getById(itdmWeituoYiju.getId());
		if( !itdmWeituoYiju1.getName().equals(itdmWeituoYiju.getName()) ){
			Map<String, Object> columnMap = new HashMap<>();
			columnMap.put("name",itdmWeituoYiju.getName());
			List<ItdmWeituoYiju> list = itdmWeituoYijuService.listByMap(columnMap);
			if(list!=null && !list.isEmpty()){
				return Result.error("该检测依据 "+ itdmWeituoYiju.getName()+ " 已存在！");
			}
		}
		itdmWeituoYijuService.updateById(itdmWeituoYiju);
		return Result.OK("编辑成功!");
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "委托依据-通过id删除")
	@ApiOperation(value="委托依据-通过id删除", notes="委托依据-通过id删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_weituo_yiju:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		itdmWeituoYijuService.removeById(id);
		return Result.OK("删除成功!");
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "委托依据-批量删除")
	@ApiOperation(value="委托依据-批量删除", notes="委托依据-批量删除")
	//@RequiresPermissions("org.jeecg.modules:itdm_weituo_yiju:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.itdmWeituoYijuService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "委托依据-通过id查询")
	@ApiOperation(value="委托依据-通过id查询", notes="委托依据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ItdmWeituoYiju> queryById(@RequestParam(name="id",required=true) String id) {
		ItdmWeituoYiju itdmWeituoYiju = itdmWeituoYijuService.getById(id);
		if(itdmWeituoYiju==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(itdmWeituoYiju);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param itdmWeituoYiju
    */
    //@RequiresPermissions("org.jeecg.modules:itdm_weituo_yiju:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ItdmWeituoYiju itdmWeituoYiju) {
        return super.exportXls(request, itdmWeituoYiju, ItdmWeituoYiju.class, "委托依据");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    //@RequiresPermissions("itdm_weituo_yiju:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, ItdmWeituoYiju.class);
    }

    /**
    *   author: dzc
    *   version: 1.0
    *   des: 检测依据查询所有的记录，不进行分页
    *   date: 2023/11/9
    */
	 @ApiOperation(value="委托依据-全部数据查询", notes="委托依据-全部数据查询")
	 @GetMapping(value = "/getAllList")
	 public Result<List<ItdmWeituoYiju>> getAllList(){
		 List<ItdmWeituoYiju> list = itdmWeituoYijuService.list();
		 return Result.OK(list);
	 }

}
